# Copyright, 2017, by Samuel G. D. Williams. <http://www.codeotaku.com>
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

module Utopia
	module Gallery
		# Represents a single unit of media, e.g. a video or image.
		class Media
			# @param path [String] The full path to the media asset.
			def initialize(path, metadata)
				@path = path
				@metadata = metadata
			end
			
			attr :path
			attr :metadata
			
			def [] key
				@metadata[key.to_s]
			end
			
			def caption
				@metadata['caption']
			end
			
			def to_s
				caption || File.basename(@path, ".*")
			end
			
			ORDER_KEY = 'order'.freeze
			
			def <=> other
				if a = self[ORDER_KEY] and b = other[ORDER_KEY]
					a <=> b
				else
					self.path <=> other.path
				end
			end
		end
	end
end
